'use strict';

/**
 * Dependencies
 */
var JOSEHeaderSchema = require('./JOSEHeaderSchema');

var _require = require('@trust/json-document'),
    JSONSchema = _require.JSONSchema;

/**
 * JWECompactSerializationSchema
 *
 * JSON Web Encryption (JWE)
 * https://tools.ietf.org/html/rfc7516#section-7.1
 *
 * 7.1.  JWE Compact Serialization
 */


var JWECompactSerializationSchema = new JSONSchema({
  type: 'object',
  additionalProperties: false,
  properties: {
    protected: JOSEHeaderSchema,
    encrypted_key: { type: 'string', format: 'base64url' },
    iv: { type: 'string', format: 'base64url' },
    ciphertext: { type: 'string', format: 'base64url' },
    tag: { type: 'string', format: 'base64url' }
  }
});

/**
 * Export
 */
module.exports = JWECompactSerializationSchema;