'use strict';

/**
 * Dependencies
 */
var JOSEHeaderSchema = require('./JOSEHeaderSchema');

var _require = require('@trust/json-document'),
    JSONSchema = _require.JSONSchema;

/**
 * JWEJSONSerializationSchema
 *
 * JSON Web Encryption (JWE)
 * https://tools.ietf.org/html/rfc7516#section-7.2
 *
 * 7.2.  JWE JSON Serialization
 *
 *
 * 7.2.1.  General JWE JSON Serialization Syntax
 */


var JWEJSONSerializationSchema = new JSONSchema({
  type: 'object',
  additionalProperties: false,
  properties: {
    protected: JOSEHeaderSchema,
    unprotected: JOSEHeaderSchema,
    iv: { type: 'string', format: 'base64url' },
    aad: { type: 'string', format: 'base64url' },
    ciphertext: { type: 'string', format: 'base64url' },
    tag: { type: 'string', format: 'base64url' },
    recipients: {
      type: 'array',
      items: [{
        type: 'object',
        properties: {
          header: JOSEHeaderSchema,
          encrypted_key: { type: 'string', format: 'base64url' }
        }
      }]
    }
  }
});

/**
 * Export
 */
//module.exports = JWEJSONSerializationSchema