'use strict';

/**
 * Dependencies
 */
var JOSEHeaderSchema = require('./JOSEHeaderSchema');
var JWTClaimsSetSchema = require('./JWTClaimsSetSchema');

var _require = require('@trust/json-document'),
    JSONSchema = _require.JSONSchema;

/**
 * JWSFlattenedSerializationSchema
 *
 * JSON Web Signature (JWS)
 * https://tools.ietf.org/html/rfc7515#section-7.2.2
 *
 * 7.2.2.  Flattened JWS JSON Serialization Syntax
 */


var JWSFlattenedSerializationSchema = new JSONSchema({
  type: 'object',
  additionalProperties: false,
  properties: {
    payload: JWTClaimsSetSchema,
    protected: JOSEHeaderSchema,
    header: JOSEHeaderSchema,
    signature: {
      type: 'string',
      format: 'base64url'
    }
  }
});

/**
 * Export
 */
module.exports = JWSFlattenedSerializationSchema;