'use strict';

/**
 * Dependencies
 */
var JOSEHeaderSchema = require('./JOSEHeaderSchema');
var JWTClaimsSetSchema = require('./JWTClaimsSetSchema');

var _require = require('@trust/json-document'),
    JSONSchema = _require.JSONSchema;

/**
 * JWSJSONSerializationSchema
 *
 * JSON Web Signature (JWS)
 * https://tools.ietf.org/html/rfc7515#section-7.2.1
 *
 * 7.2.1.  General JWS JSON Serialization Syntax
 */


var JWSJSONSerializationSchema = new JSONSchema({
  type: 'object',
  additionalProperties: false,
  properties: {
    payload: JWTClaimsSetSchema,
    signatures: {
      type: 'array',
      items: [{
        type: 'object',
        properties: {
          protected: JOSEHeaderSchema,
          header: JOSEHeaderSchema,
          signature: {
            type: 'string',
            format: 'base64url'
          }
        }
      }]
    }
  }
});

/**
 * Export
 */
module.exports = JWSJSONSerializationSchema;